@@ -1,5 +1,5 @@
 /* code generation
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005
    Craig Franklin
 
 This file is part of gputils.
@@ -23,7 +23,6 @@ Boston, MA 02111-1307, USA.  */
 
 #include "libgputils.h"
 #include "gpal.h"
-#include "symbol.h"
 #include "analyze.h" 
 #include "codegen.h"
 #include "codegen14.h"
@@ -39,39 +38,53 @@ static int codegen_working_bytes;
 static enum size_tag codegen_size;
 int codegen_bytes;
 
-static struct function_pointer_struct *func_ptr;
+target_type *target = NULL;
 
 /* code generation function pointer */
 typedef void code_func(enum node_op op, 
                        enum size_tag size,
                        gp_boolean is_const,
                        int value,
-                       char *name);
+                       char *name,
+                       char *bank_addr);
 
-/* data movement function pointers */
+/* unop code generation function pointer */
 typedef void code_unop(enum node_op op,
                        gp_boolean direct,
                        char *name,
+                       char *bank_addr,
                        enum size_tag size,
-                       int offset,
-                       gp_boolean add_banksel);
+                       int offset);
 
 /* load constant function pointer */
 typedef void data_func1(int value, enum size_tag size);
 
 /* data movement function pointers */
 typedef void data_func2(char *name,
+                        char *bank_addr,
                         enum size_tag size,
-                        int offset,
-                        gp_boolean add_banksel);
+                        int offset);
 
-#define CODEGEN (*(code_func*)func_ptr->codegen)
-#define UNOPGEN (*(code_unop*)func_ptr->unopgen)
-#define LOAD_CONSTANT (*(data_func1*)func_ptr->load_constant)
-#define LOAD_FILE (*(data_func2*)func_ptr->load_file)
-#define STORE_FILE (*(data_func2*)func_ptr->store_file)
-#define LOAD_INDIRECT (*(data_func2*)func_ptr->load_indirect)
-#define STORE_INDIRECT (*(data_func2*)func_ptr->store_indirect)
+/* load address function pointer */
+typedef void data_func3(char *name, enum size_tag size);
+
+/* variable operation function pointers */
+typedef void var_func(struct variable *var);
+
+#define CODEGEN (*(code_func*)target->codegen)
+#define UNOPGEN (*(code_unop*)target->unopgen)
+#define LOAD_CONSTANT (*(data_func1*)target->load_constant)
+#define LOAD_FILE (*(data_func2*)target->load_file)
+#define STORE_FILE (*(data_func2*)target->store_file)
+#define LOAD_ADDRESS (*(data_func3*)target->load_address)
+#define LOAD_FSR (*(var_func*)target->load_fsr)
+#define OFFSET_FSR (*(var_func*)target->offset_fsr)
+#define LOAD_INDIRECT (*(data_func2*)target->load_indirect)
+#define STORE_INDIRECT (*(data_func2*)target->store_indirect)
+#define RESET_VECTOR (*(var_func*)target->reset_vector)
+#define INT_VECTOR (*(var_func*)target->interrupt_vector)
+
+#define LOCAL_BANK FILE_DATA_ADDR(state.module)
 
 /****************************************************************************/
 /* Common Assembly Functions                                                */
@@ -110,7 +123,7 @@ codegen_write_comment(const char *format, ...)
 void
 codegen_line_number(tree *node)
 {
-  fprintf(state.output.f, "#line %d\n", node->line_number);
+  fprintf(state.output.f, "  .line %d\n", node->line_number);
 }
 
 char *
@@ -124,6 +137,11 @@ codegen_next_label(void)
 void
 codegen_write_label(char *label)
 {
+
+  /* Clear the bank setting, because current bank is unknown. */
+  state.current_bank = NULL;
+  state.current_ibank = NULL;
+
   fprintf(state.output.f, "%s:\n", label);
 }
 
@@ -133,6 +151,83 @@ codegen_write_equ(char *label, int value)
   fprintf(state.output.f, "%-30s equ %#x\n", label, value);
 }
 
+void
+codegen_set_bank(char *bank_name)
+{
+  struct variable *var;
+
+  assert(bank_name != NULL);
+
+  if ((state.current_bank == NULL) || 
+      (strcasecmp(bank_name, state.current_bank) != 0)) {
+    codegen_write_asm("banksel %s", bank_name);
+    state.current_bank = bank_name;
+    var = get_global(bank_name);
+    if (var) {
+      var->is_used = true;
+    }
+  }
+
+}
+
+void
+codegen_set_ibank(char *ibank_name)
+{
+  struct variable *var;
+
+  assert(ibank_name != NULL);
+
+  if ((target->use_bankisel) &&
+      ((state.current_ibank == NULL) || 
+       (strcasecmp(ibank_name, state.current_ibank) != 0))) {
+    codegen_write_asm("bankisel %s", ibank_name);
+    state.current_ibank = ibank_name;
+    var = get_global(ibank_name);
+    if (var) {
+      var->is_used = true;
+    }
+  }
+
+}
+
+void
+codegen_set_page(char *page_name)
+{
+  struct variable *var;
+
+  assert(page_name != NULL);
+
+  if ((target->use_pagesel) &&
+      ((state.current_page == NULL) || 
+       (strcasecmp(page_name, state.current_page) != 0))) {
+    codegen_write_asm("pagesel %s", page_name);
+    state.current_page = page_name;
+    var = get_global(page_name);
+    if (var) {
+      var->is_used = true;
+    }
+  }
+
+}
+
+/* Generate offset buffer string.  Static buffer used, so not thread-safe */
+
+char *
+codegen_get_offset_buffer(int offset)
+{
+  static char buffer[64];
+
+  if (offset == 0) {
+    buffer[0] = '\0';
+  } else if (offset < 0) {
+    snprintf(buffer, sizeof(buffer), " - %#x", -offset);
+  } else {
+    snprintf(buffer, sizeof(buffer), " + %#x", offset);
+  }
+
+  return buffer;
+}
+
 /****************************************************************************/
 /* Common directives and instructions                                       */
 /****************************************************************************/
@@ -167,51 +262,44 @@ codegen_assembly(tree *assembly)
 /* Memory access                                                            */
 /****************************************************************************/
 
+/* load the address for indirect access */
+
+void
+codegen_load_indirect(struct variable *var)
+{
+
+  LOAD_FSR(var);
+
+  return;
+}
+
 /* calculate the byte address of the data and put it in FSR */
 
 void
-codegen_indirect(tree *offset,
-                 struct variable *var,
-                 int element_size,
-                 gp_boolean new_expr)
+codegen_calc_indirect(tree *offset,
+                      struct variable *var,
+                      int element_size,
+                      gp_boolean new_expr)
 {
 
   /* indirect access */
   if (new_expr) {
-    codegen_expr(offset, state.pointer_size);
+    codegen_expr(offset, target->data_ptr_size);
   } else {
     gen_expr(offset);
   }
+
   if (var->type->start != 0) {
     /* shift the offset for the first element */
-    CODEGEN(op_add, state.pointer_size, true, 0 - var->type->start, NULL);
+    CODEGEN(op_add, target->data_ptr_size, true, 0 - var->type->start, NULL, NULL);
   }
+
   if (element_size != 1) {
     /* scale the offset by the element size */
-    CODEGEN(op_mult, state.pointer_size, true, element_size, NULL);
-  }
-
-  switch (state.class) {
-  case PROC_CLASS_PIC14:
-    codegen_write_asm("addlw %s", var->name); 
-    codegen_write_asm("banksel FSR");
-    codegen_write_asm("movwf FSR"); 
-    codegen_write_asm("banksel %s", LOCAL_DATA_LABEL);
-    break;
-  case PROC_CLASS_PIC16E:
-    codegen_write_asm("movf %s, w", WORKING_LABEL);
-    codegen_write_asm("addlw LOW(%s)", var->name); 
-    codegen_write_asm("movff WREG, FSR0L");
-    codegen_write_asm("movf %s + 1, w", WORKING_LABEL);
-    codegen_write_asm("btfsc STATUS, C");
-    codegen_write_asm("addlw 1"); 
-    codegen_write_asm("addlw HIGH(%s)", var->name); 
-    codegen_write_asm("movff WREG, FSR0H");
-    break;
-  default:
-    assert(0);
+    CODEGEN(op_mult, target->data_ptr_size, true, element_size, NULL, NULL);
   }
 
+  OFFSET_FSR(var);
 
   return;
 }
@@ -223,38 +311,27 @@ codegen_load_file(tree *symbol, struct variable *var)
 {
   int offset;
   int element_size;
+  char *bank_addr = var_bank(var);
 
-  if ((symbol->tag == node_symbol) && (SYM_OFST(symbol))) {
+  if ((symbol->tag == tag_symbol) && (SYM_OFST(symbol))) {
     /* access an array */
-    element_size = type_size(var->type->prim);
+    element_size = type_bytes(var->type->prim);
 
     if ((var) && (var->type) && (var->type->tag == type_array)) {
       if (can_evaluate(SYM_OFST(symbol), false)) {
         /* direct access */
         offset = analyze_check_array(symbol, var) * element_size;
-        if (is_far(var)) {
-          LOAD_FILE(var->name, codegen_size, offset, true);
-        } else {
-          LOAD_FILE(var->name, codegen_size, offset, false);
-        }
+        LOAD_FILE(var->name, bank_addr, codegen_size, offset);
       } else {
-        codegen_indirect(SYM_OFST(symbol), var, element_size, false);
-        if (is_far(var)) {
-          LOAD_INDIRECT(var->name, codegen_size, 0, true);
-        } else {
-          LOAD_INDIRECT(var->name, codegen_size, 0, false);
-        }
+        codegen_calc_indirect(SYM_OFST(symbol), var, element_size, false);
+        LOAD_INDIRECT(var->name, bank_addr, codegen_size, 0);
       }
     } else {
       analyze_error(symbol, "symbol %s is not an array",
                     SYM_NAME(symbol));
     }
   } else {
-    if (is_far(var)) {
-      LOAD_FILE(var->name, codegen_size, 0, true);
-    } else {
-      LOAD_FILE(var->name, codegen_size, 0, false);
-    }
+    LOAD_FILE(var->name, bank_addr, codegen_size, 0);
   }
 
   return;
@@ -264,31 +341,23 @@ codegen_load_file(tree *symbol, struct variable *var)
 
 void
 codegen_store(struct variable *var,
+              gp_boolean access,
               gp_boolean constant_offset,
               int offset,
               tree *offset_expr)
 {
+  char *bank_addr = var_bank(var);
 
-  if (offset_expr) {
+  if (access) {
+    STORE_INDIRECT(var->name, bank_addr, codegen_size, 0);
+  } else if (offset_expr) {
     if (constant_offset) {
-      if (is_far(var)) {
-        STORE_FILE(var->name, codegen_size, offset, true);
-      } else {
-        STORE_FILE(var->name, codegen_size, offset, false);
-      }    
+      STORE_FILE(var->name, bank_addr, codegen_size, offset);
     } else {
-      if (is_far(var)) {
-        STORE_INDIRECT(var->name, codegen_size, 0, true);
-      } else {
-        STORE_INDIRECT(var->name, codegen_size, 0, false);
-      }
+      STORE_INDIRECT(var->name, bank_addr, codegen_size, 0);
     }
   } else {
-    if (is_far(var)) {
-      STORE_FILE(var->name, codegen_size, 0, true);
-    } else {
-      STORE_FILE(var->name, codegen_size, 0, false);
-    }
+    STORE_FILE(var->name, bank_addr, codegen_size, 0);
   }
 
 }
@@ -307,7 +376,8 @@ gen_unop_expr(tree *expr)
     break;
   case op_not:
   case op_neg:
-    CODEGEN(expr->value.unop.op, codegen_size, true, 0, NULL);
+  case op_com:
+    CODEGEN(expr->value.unop.op, codegen_size, true, 0, NULL, NULL);
     break;
   default:
     assert(0);
@@ -321,38 +391,39 @@ gen_binop_expr(enum node_op op, tree *p0, tree *p1)
   char *reg1 = NULL;
   char *reg2 = NULL;
   struct variable *var;
+  char *local_bank_addr = LOCAL_BANK;
 
   gen_expr(p1);
 
-  if (p0->tag == node_call) {
+  if (p0->tag == tag_call) {
     reg1 = codegen_get_temp(codegen_size);
-    STORE_FILE(reg1, codegen_size, 0, false);
+    STORE_FILE(reg1, local_bank_addr, codegen_size, 0);
     analyze_call(p0, true, codegen_size);    
-    CODEGEN(op, codegen_size, false, 0, reg1);
-  } else if (p0->tag == node_constant) {
-    CODEGEN(op, codegen_size, true, p0->value.constant, NULL);
-  } else if (p0->tag == node_symbol) { 
+    CODEGEN(op, codegen_size, false, 0, reg1, local_bank_addr);
+  } else if (p0->tag == tag_constant) {
+    CODEGEN(op, codegen_size, true, p0->value.constant, NULL, NULL);
+  } else if (p0->tag == tag_symbol) { 
     var = get_global(SYM_NAME(p0));
     if (var->tag == sym_const) {
-      CODEGEN(op, codegen_size, true, var->value, NULL);
+      CODEGEN(op, codegen_size, true, var->value, NULL, NULL);
     } else if (SYM_OFST(p0)) {
       /* it is a complex expression, so save temp data */
       reg1 = codegen_get_temp(codegen_size);
       reg2 = codegen_get_temp(codegen_size);
-      STORE_FILE(reg1, codegen_size, 0, false);
+      STORE_FILE(reg1, local_bank_addr, codegen_size, 0);
       codegen_load_file(p0, var);
-      STORE_FILE(reg2, codegen_size, 0, false);
-      LOAD_FILE(reg1, codegen_size, 0, false);
-      CODEGEN(op, codegen_size, false, 0, reg2);
+      STORE_FILE(reg2, local_bank_addr, codegen_size, 0);
+      LOAD_FILE(reg1, local_bank_addr, codegen_size, 0);
+      CODEGEN(op, codegen_size, false, 0, reg2, local_bank_addr);
     } else {
-      CODEGEN(op, codegen_size, false, 0, var->name);
+      CODEGEN(op, codegen_size, false, 0, var->name, var_bank(var));
     }
   } else {
     /* it is a complex expression so save temp data */
     reg1 = codegen_get_temp(codegen_size);
-    STORE_FILE(reg1, codegen_size, 0, false);
+    STORE_FILE(reg1, local_bank_addr, codegen_size, 0);
     gen_expr(p0);
-    CODEGEN(op, codegen_size, false, 0, reg1);
+    CODEGEN(op, codegen_size, false, 0, reg1, local_bank_addr);
   }
 
   if (reg1)
@@ -366,30 +437,45 @@ static void
 gen_expr(tree *expr)
 {
   struct variable *var;
+  char *bank_addr;
 
   switch(expr->tag) {
-  case node_arg:
+  case tag_arg:
     var = get_global(ARG_NAME(expr));
     codegen_load_file(expr, var);
     break;
-  case node_call:
+  case tag_attrib:
+    var = get_global(ATTRIB_NAME(expr));
+    if (ATTRIB_TYPE(expr) == attrib_access) {
+      LOAD_FSR(var);
+      bank_addr = var_bank(var);
+      LOAD_INDIRECT(var->name, bank_addr, codegen_size, 0);
+    } else if (ATTRIB_TYPE(expr) == attrib_address) {
+      LOAD_ADDRESS(var->name, codegen_size);
+    } else {
+      LOAD_CONSTANT(maybe_evaluate(expr), codegen_size);
+    }
+    break;
+  case tag_call:
     analyze_call(expr, true, codegen_size);    
     break;
-  case node_constant:
+  case tag_constant:
     LOAD_CONSTANT(expr->value.constant, codegen_size);
     break;
-  case node_symbol:
+  case tag_symbol:
     var = get_global(SYM_NAME(expr));
     if (var->tag == sym_const) {
       LOAD_CONSTANT(var->value, codegen_size); 
+    } else if (var->tag == sym_alias) {
+      gen_expr(var->node);
     } else {
       codegen_load_file(expr, var);
     }
     break;
-  case node_unop:
+  case tag_unop:
     gen_unop_expr(expr);
     break;
-  case node_binop:
+  case tag_binop:
     if (expr->value.binop.op == op_assign) {
       analyze_error(expr, "assign operator = should be equal operator ==");
     } else if ((expr->value.binop.op == op_lsh) ||
@@ -417,7 +503,7 @@ static int
 codegen_setup(enum size_tag size)
 {
   codegen_size = size;
-  codegen_bytes = prim_size(size);
+  codegen_bytes = prim_bytes(size);
 
   if (codegen_bytes > codegen_working_bytes) {
     codegen_working_bytes = codegen_bytes;
@@ -471,29 +557,18 @@ codegen_unop(struct variable *var,
              tree *unop,
              enum size_tag size)
 {
+  char *bank_addr = var_bank(var);
 
   temp_number = codegen_setup(size);
 
   if (offset_expr) {
     if (constant_offset) {
-      if (is_far(var)) {
-        UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, offset, true);
-      } else {
-        UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, offset, false);
-      }    
+      UNOPGEN(UNOP_OP(unop), true, var->name, bank_addr, codegen_size, offset);
     } else {
-      if (is_far(var)) {
-        UNOPGEN(UNOP_OP(unop), false, var->name, codegen_size, 0, true);
-      } else {
-        UNOPGEN(UNOP_OP(unop), false, var->name, codegen_size, 0, false);
-      }
+      UNOPGEN(UNOP_OP(unop), false, var->name, bank_addr, codegen_size, 0);
     }
   } else {
-    if (is_far(var)) {
-      UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, 0, true);
-    } else {
-      UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, 0, false);
-    }
+    UNOPGEN(UNOP_OP(unop), true, var->name, bank_addr, codegen_size, 0);
   }
 
   if (temp_number > max_temp_number)
@@ -538,36 +613,43 @@ codegen_init_data()
 {
   codegen_write_comment("declarations");
 
-  if (state.section.udata) {
-    if (state.section.udata_addr_valid) {
+  if (state.section.data) {
+    if (state.section.data_addr_valid) {
       fprintf(state.output.f, "%s udata %#x\n", 
-              state.section.udata,
-              state.section.udata_addr);
+              state.section.data,
+              state.section.data_addr);
     } else {
-      fprintf(state.output.f, "%s udata\n", state.section.udata);
+      fprintf(state.output.f, "%s udata\n", state.section.data);
     }
   } else {
-    if (state.section.udata_addr_valid) {
+    if (state.section.data_addr_valid) {
       fprintf(state.output.f, ".udata_%s udata %#x\n",
               FILE_NAME(state.module),
-              state.section.udata_addr);
+              state.section.data_addr);
     } else {
       fprintf(state.output.f, ".udata_%s udata\n", FILE_NAME(state.module));
     }  
   }
 
-  codegen_write_label(LOCAL_DATA_LABEL);
-  
+  codegen_write_label(LOCAL_BANK);
+  codegen_write_asm("global %s", LOCAL_BANK);
 }
 
 void
-codegen_write_data(char *label, int size, enum node_storage storage)
+codegen_write_data(struct variable *var)
 {
 
-  fprintf(state.output.f, "%s res %i\n", label, size);
+  fprintf(state.output.f, "  .def %s, ", var->name);
 
-  if (storage == storage_public)
-    codegen_write_asm("global %s", label);
+  if (var->storage == storage_public) {
+    fprintf(state.output.f, "global, ");
+  } else {
+    fprintf(state.output.f, "static, ");
+  }
+
+  fprintf(state.output.f, "size=%#x, type=%#x\n",
+          type_bytes(var->type),
+          var_coff_type(var));
 
 }
 
@@ -597,7 +679,7 @@ codegen_get_temp(enum size_tag size)
            FILE_NAME(state.module),
            temp_number);
 
-  temp_number += prim_size(size);
+  temp_number += prim_bytes(size);
 
   return strdup(temp_name);
 }
@@ -607,16 +689,17 @@ codegen_get_temp(enum size_tag size)
 /****************************************************************************/
 
 void
-codegen_init_asm()
+codegen_init_asm(tree *module)
 {
+  char asmfilename[BUFSIZ];
   char buffer[BUFSIZ];
 
   /* open output filename */
-  strncpy(state.asmfilename, state.basefilename, sizeof(state.asmfilename));
-  strncat(state.asmfilename, ".asm", sizeof(state.asmfilename));
-  state.output.f = fopen(state.asmfilename, "w");
+  strncpy(asmfilename, FILE_NAME(module), sizeof(asmfilename));
+  strncat(asmfilename, ".asm", sizeof(asmfilename));
+  state.output.f = fopen(asmfilename, "w");
   if (state.output.f == NULL) {
-    perror(state.asmfilename);
+    perror(asmfilename);
     exit(1);
   }
 
@@ -624,16 +707,19 @@ codegen_init_asm()
   max_temp_number = 0;
   codegen_working_bytes = 1;
   codegen_size = size_uint8;
-  codegen_bytes = prim_size(codegen_size);
+  codegen_bytes = prim_bytes(codegen_size);
 
   gp_date_string(buffer, sizeof(buffer));
 
-  fprintf(state.output.f, "; %s\n", state.asmfilename);
+  fprintf(state.output.f, "; %s\n", asmfilename);
   fprintf(state.output.f, "; generated by %s on %s\n\n",
           GPAL_VERSION_STRING,
           buffer);
 
-  fprintf(state.output.f, "#file \"%s\"\n\n", state.srcfilename);
+  fprintf(state.output.f, "  .ident \"%s\"\n\n", GPAL_VERSION_STRING);
+
+  fprintf(state.output.f, "  .file \"%s\"\n\n",
+          get_file_name(module->file_id));
 
   if (state.processor_chosen) {
     fprintf(state.output.f, "  processor %s\n", 
@@ -655,13 +741,16 @@ codegen_init_asm()
   } else {
     if (state.section.code_addr_valid) {
       fprintf(state.output.f, ".code_%s code %#x\n", 
-              FILE_NAME(state.module),
+              FILE_NAME(module),
               state.section.code_addr);
     } else {
-      fprintf(state.output.f, ".code_%s code\n", FILE_NAME(state.module));
+      fprintf(state.output.f, ".code_%s code\n", FILE_NAME(module));
     }
   }
 
+  codegen_write_label(FILE_CODE_ADDR(module));
+  codegen_write_asm("global %s", FILE_CODE_ADDR(module));
+
   return;
 }
 
@@ -671,47 +760,16 @@ codegen_close_asm(void)
   struct variable *var;
 
   var = get_global("main");
-  if ((var) && (var->node->tag == node_proc)) {
-    /* a procedure named "main" exists so add the startup code */
-    codegen_write_comment("startup and interrupt vectors");
-    fprintf(state.output.f, "RESET_VECTOR code 0x0\n");
-    codegen_line_number(var->node);
-    fprintf(state.output.f, "  movlw high %s\n", var->name);
-    fprintf(state.output.f, "  movwf PCLATH\n");
-    fprintf(state.output.f, "  goto %s\n\n", var->name);
+  if ((var) && (var->node->tag == tag_subprogram)) {
+    RESET_VECTOR(var);
   }
 
   var = get_global("isr");
-  if ((var) && (var->node->tag == node_proc)) {
-    fprintf(state.output.f, "INT_VECTOR code 0x4\n");
-    codegen_write_comment("store wreg and status");
-    fprintf(state.output.f, "  movwf w_temp\n");
-    fprintf(state.output.f, "  movf STATUS, w\n");
-    fprintf(state.output.f, "  movwf status_temp\n");
-    fprintf(state.output.f, "  movf PCLATH, w\n");
-    fprintf(state.output.f, "  movwf pclath_temp\n");
-
-    codegen_write_comment("call the interrupt routine");
-    fprintf(state.output.f, "  pagesel %s\n", var->name);
-    fprintf(state.output.f, "  call %s\n", var->name);
-
-    codegen_write_comment("restore wreg and status");
-    fprintf(state.output.f, "  movf pclath_temp, w\n");
-    fprintf(state.output.f, "  movwf PCLATH\n");
-    fprintf(state.output.f, "  movf status_temp, w\n");
-    fprintf(state.output.f, "  movwf STATUS\n");
-    fprintf(state.output.f, "  swapf w_temp, f\n");
-    fprintf(state.output.f, "  swapf w_temp, w\n");
-    fprintf(state.output.f, "  retfie\n\n");
-    
-    fprintf(state.output.f, "INT_VAR udata_shr\n");
-    codegen_write_comment("FIXME: not all processors have shared memory");
-    fprintf(state.output.f, "w_temp res 1\n");
-    fprintf(state.output.f, "status_temp res 1\n\n");
-    fprintf(state.output.f, "pclath_temp res 1\n\n");
+  if ((var) && (var->node->tag == tag_subprogram)) {
+    INT_VECTOR(var);
   }
 
-  fprintf(state.output.f, "#eof\n\n");
+  fprintf(state.output.f, "  .eof\n\n");
   fprintf(state.output.f, "  end\n\n");
 
   fclose(state.output.f);
@@ -728,18 +786,16 @@ codegen_select(tree *expr)
   case PROC_CLASS_GENERIC:
   case PROC_CLASS_PIC12:
   case PROC_CLASS_SX:
-    analyze_error(expr, "unsupported processor class");
+    analyze_error(expr, "unsupported device");
     break;
   case PROC_CLASS_PIC14:
-    func_ptr = &codegen14_func;
-    state.pointer_size = size_uint8;
+    target = &pic14;
     break;
   case PROC_CLASS_PIC16:
-    analyze_error(expr, "unsupported processor class");
+    analyze_error(expr, "unsupported device");
     break;
   case PROC_CLASS_PIC16E:
-    func_ptr = &codegen16e_func;
-    state.pointer_size = size_uint16;
+    target = &pic16e;
     break;
   default:
     assert(0);
