@@ -400,6 +400,20 @@ static gpasmVal do_badram(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_badrom(gpasmVal r,
+			  char *name,
+			  int arity,
+			  struct pnode *parms)
+{
+
+  state.lst.line.linetype = dir;
+
+  /* FIXME: implement this directive */
+  gpwarning(GPW_UNKNOWN, "gpasm doesn't support the badrom directive yet"); 
+
+  return r;
+}
+
 static void
 set_bankisel(int address)
 {
@@ -670,6 +684,9 @@ static gpasmVal do_config(gpasmVal r,
       i_memory_put(state.c_memory, ca, MEM_USED_MASK | value);
       /* FIXME: need line_number? this one will be wrong coff_linenum(1) */
     }
+
+    /* force the section to end */
+    state.obj.section = NULL;
   }
         
   return r;
@@ -1324,18 +1341,16 @@ static gpasmVal do_extern(gpasmVal r,
 		          int arity,
 		          struct pnode *parms)
 {
-  struct pnode *p;
+  char *p;
   state.lst.line.linetype = dir;
   
   if (state.mode == absolute) {
     gperror(GPE_OBJECT_ONLY, NULL);
   } else {
     for (; parms; parms = TAIL(parms)) {
-      p = HEAD(parms);
-      if (p->tag == symbol) {
-        set_global(p->value.symbol, 0, PERMANENT, gvt_extern);
-      } else {
-        gperror(GPE_ILLEGAL_ARGU, NULL);
+      p = maybe_evaluate_concat(HEAD(parms));
+      if (p) {
+        set_global(p, 0, PERMANENT, gvt_extern);
       }
     }
   }
@@ -1398,7 +1413,7 @@ static gpasmVal do_global(gpasmVal r,
 		          int arity,
 		          struct pnode *parms)
 {
-  struct pnode *p;
+  char *p;
   char buf[BUFSIZ];
   struct symbol *s;
   struct variable *var;
@@ -1409,14 +1424,14 @@ static gpasmVal do_global(gpasmVal r,
     gperror(GPE_OBJECT_ONLY, NULL);
   } else {
     for (; parms; parms = TAIL(parms)) {
-      p = HEAD(parms);
-      if (p->tag == symbol) {
-        s = get_symbol(state.stTop, p->value.symbol);
+      p = maybe_evaluate_concat(HEAD(parms));
+      if (p) {
+        s = get_symbol(state.stTop, p);
         if (s == NULL) {
           snprintf(buf,
                    sizeof(buf),
                    "Symbol not previously defined (%s).",
-                   p->value.symbol);
+                   p);
           gperror(GPE_NOSYM, buf);    
 	} else {
           var = get_symbol_annotation(s);
@@ -1424,7 +1439,7 @@ static gpasmVal do_global(gpasmVal r,
             snprintf(buf,
                      sizeof(buf),
                      "Symbol not assigned a value (%s).",
-                     p->value.symbol);
+                     p);
             gpwarning(GPW_UNKNOWN, buf);    
           } else {
             if ((var->previous_type == gvt_address) || 
@@ -1438,7 +1453,7 @@ static gpasmVal do_global(gpasmVal r,
               snprintf(buf,
                        sizeof(buf),
                        "Operand must be an address label (%s).", 
-                       p->value.symbol);
+                       p);
               gperror(GPE_MUST_BE_LABEL, buf);    
             } 
           }         
@@ -1913,6 +1928,20 @@ static gpasmVal do_maxram(gpasmVal r,
   return r;
 }
 
+static gpasmVal do_maxrom(gpasmVal r,
+			  char *name,
+			  int arity,
+			  struct pnode *parms)
+{
+
+  state.lst.line.linetype = dir;
+
+  /* FIXME: implement this directive */
+  gpwarning(GPW_UNKNOWN, "gpasm doesn't support the maxrom directive yet"); 
+
+  return r;
+}
+
 static gpasmVal do_macro(gpasmVal r,
 			 char *name,
 			 int arity,
@@ -3452,10 +3481,12 @@ const int num_op_0 = TABLE_SIZE(op_0);
 
 struct insn op_1[] = {
   { "__badram",   0, (long int)do_badram,    INSN_CLASS_FUNC,   0 },
+  { "__badrom",   0, (long int)do_badrom,    INSN_CLASS_FUNC,   0 },
   { "__config",   0, (long int)do_config,    INSN_CLASS_FUNC,   0 },
   { "__fuses",    0, (long int)do_config,    INSN_CLASS_FUNC,   0 },
   { "__idlocs",   0, (long int)do_idlocs,    INSN_CLASS_FUNC,   0 },
   { "__maxram",   0, (long int)do_maxram,    INSN_CLASS_FUNC,   0 },
+  { "__maxrom",   0, (long int)do_maxrom,    INSN_CLASS_FUNC,   0 },
   { "bankisel",   0, (long int)do_bankisel,  INSN_CLASS_FUNC,   0 },
   { "banksel",    0, (long int)do_banksel,   INSN_CLASS_FUNC,   0 },
   { "data",       0, (long int)do_data,      INSN_CLASS_FUNC,   0 },
