@@ -1,5 +1,5 @@
 /* top level functions for gpasm
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman, Craig Franklin
 
 This file is part of gputils.
@@ -33,13 +33,13 @@ Boston, MA 02111-1307, USA.  */
 
 struct gpasm_state state;
 
-static int cmd_processor = 0;
+static gp_boolean cmd_processor = false;
 static char *processor_name = NULL;
 
 int yyparse(void);
 extern int yydebug;
 
-#define GET_OPTIONS "?D:I:La:cde:ghilmno:p:qr:vw:"
+#define GET_OPTIONS "?D:I:La:cde:ghilmno:p:qr:uvw:y"
 
 /* Used: acdDehiIlmopqrwv */
 static struct option longopts[] =
@@ -61,8 +61,10 @@ static struct option longopts[] =
   { "processor",   1, 0, 'p' },
   { "quiet",       0, 0, 'q' },
   { "radix",       1, 0, 'r' },
+  { "absolute",    0, 0, 'u' },
   { "version",     0, 0, 'v' },
   { "warning",     1, 0, 'w' },
+  { "extended",    0, 0, 'y' },
   { 0, 0, 0, 0 }
 };
 
@@ -74,29 +76,31 @@ init(void)
 
   /* restore gpasm to its initialized state */
   state.mode = absolute;
+  state.extended_pic16e = false;
 
   state.radix = 16;
   state.hex_format = inhx32;
-  state.case_insensitive = 0;
-  state.quiet = 0;
+  state.case_insensitive = false;
+  state.quiet = false;
+  state.use_absolute_path = false;
   state.error_level = 0;
-  state.debug_info = 0;
+  state.debug_info = false;
   state.path_num = 0;
 
-  state.cmd_line.radix = 0;
-  state.cmd_line.hex_format = 0;
-  state.cmd_line.error_level = 0;
-  state.cmd_line.macro_expand = 0;
-  state.cmd_line.processor = 0;
-  state.cmd_line.lst_force = 0;
+  state.cmd_line.radix = false;
+  state.cmd_line.hex_format = false;
+  state.cmd_line.error_level = false;
+  state.cmd_line.macro_expand = false;
+  state.cmd_line.processor = false;
+  state.cmd_line.lst_force = false;
 
   state.pass = 0;
   state.org = 0;
-  state.dos_newlines = 0;
-  state.memory_dump = 0;
-  state.found_config = 0;
-  state.found_devid = 0;
-  state.found_idlocs = 0;
+  state.dos_newlines = false;
+  state.memory_dump = false;
+  state.found_config = false;
+  state.found_devid = false;
+  state.found_idlocs = false;
   state.register_block = false;
   state.maxram = (MAX_RAM - 1);
 
@@ -162,8 +166,10 @@ show_usage(void)
   printf("  -p PROC, --processor PROC      Select processor.\n");
   printf("  -q, --quiet                    Quiet.\n");
   printf("  -r RADIX, --radix RADIX        Select radix. [hex]\n");
-  printf("  -w [0|1|2], --warning [0|1|2]  Set message level. [0]\n");
+  printf("  -u, --absolute                 Use absolute pathes. \n");
   printf("  -v, --version                  Show version.\n");
+  printf("  -w [0|1|2], --warning [0|1|2]  Set message level. [0]\n");
+  printf("  -y, --extended                 Enable 18xx extended mode.\n");
   printf("\n");
 #ifdef USE_DEFAULT_PATHS
   if (gp_header_path) {
@@ -174,7 +180,7 @@ show_usage(void)
   printf("\n");
 #endif
   printf("Report bugs to:\n");
-  printf("%s\n", BUG_REPORT_URL);
+  printf("%s\n", PACKAGE_BUGREPORT);
   exit(0);
 }
 
@@ -184,15 +190,15 @@ process_args( int argc, char *argv[])
   extern char *optarg;
   extern int optind;
   int c;
-  int usage = 0;
+  gp_boolean usage = false;
   char *pc;
 
-  /* Scan through the options for the -c flag.  It must be set before the 
+  /* Scan through the options for the -i flag.  It must be set before the 
      defines are read */
   while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, 0)) != EOF) {
     switch (c) {
     case 'i':
-      state.case_insensitive = 1;
+      state.case_insensitive = true;
       break;
     }
   }
@@ -207,11 +213,11 @@ process_args( int argc, char *argv[])
     switch (c) {
     case '?':
     case 'h':
-      usage = 1;
+      usage = true;
       break;
     case 'a':
       select_hexformat(optarg);
-      state.cmd_line.hex_format = 1;
+      state.cmd_line.hex_format = true;
       break;
     case 'c':
       state.mode    = relocatable;
@@ -221,7 +227,7 @@ process_args( int argc, char *argv[])
       state.objfile = normal;
       break;
     case 'd':
-      gp_debug_disable = 0;
+      gp_debug_disable = false;
       break;
     case 'D':
       if ((optarg != NULL) && (strlen(optarg) > 0)) {
@@ -244,30 +250,30 @@ process_args( int argc, char *argv[])
       break;
     case 'e':
       select_expand(optarg);
-      state.cmd_line.macro_expand = 1;
+      state.cmd_line.macro_expand = true;
       break;
     case 'g':
-      state.debug_info = 1;
+      state.debug_info = true;
       break;
     case 'I':
       add_path(optarg);
       break;    
     case 'i':
-      state.case_insensitive = 1;
+      state.case_insensitive = true;
       break;
     case 'L':
-      state.cmd_line.lst_force = 1;
+      state.cmd_line.lst_force = true;
       break;  
     case 'l':
       gp_dump_processor_list(true, 0);
       exit(0);
       break;
     case 'm':
-      state.memory_dump = 1;
+      state.memory_dump = true;
       break;
     case 'n':
       #ifndef HAVE_DOS_BASED_FILE_SYSTEM
-        state.dos_newlines = 1;
+        state.dos_newlines = true;
       #endif
       break;
     case 'o':
@@ -278,19 +284,25 @@ process_args( int argc, char *argv[])
         *pc = 0;
       break;
     case 'p':
-      cmd_processor = 1;
+      cmd_processor = true;
       processor_name = optarg;
       break;
     case 'q':
-      state.quiet = 1;
+      state.quiet = true;
       break;
     case 'r':
       select_radix(optarg);
-      state.cmd_line.radix = 1;
+      state.cmd_line.radix = true;
+      break;
+    case 'u':
+      state.use_absolute_path = true;
       break;
     case 'w':
       select_errorlevel(atoi(optarg));
-      state.cmd_line.error_level = 1;
+      state.cmd_line.error_level = true;
+      break;
+    case 'y':
+      state.extended_pic16e = true;
       break;
     case 'v':
       fprintf(stderr, "%s\n", GPASM_VERSION_STRING);
@@ -304,7 +316,7 @@ process_args( int argc, char *argv[])
   if ((optind + 1) == argc)
     state.srcfilename = argv[optind];
   else
-    usage = 1;
+    usage = true;
 
   if (usage) {
     show_usage();
@@ -316,6 +328,10 @@ process_args( int argc, char *argv[])
     add_path(gp_header_path);
   }
 
+  if (state.use_absolute_path) {
+    state.srcfilename = gp_absolute_path(state.srcfilename);
+  }
+
 }
 
 int
@@ -336,7 +352,7 @@ assemble(void)
   }
 
   /* Builtins are always case insensitive */
-  state.stBuiltin = push_symbol_table(NULL, 1);
+  state.stBuiltin = push_symbol_table(NULL, true);
   state.stDirective = state.stBuiltin;
   state.stMacros = push_symbol_table(NULL, state.case_insensitive);
   state.stTop = 
@@ -349,7 +365,7 @@ assemble(void)
   /* the tables are built, select the processor if -p was used */
   if (cmd_processor) {
     select_processor(processor_name);
-    state.cmd_line.processor = 1;
+    state.cmd_line.processor = true;
   }
 
   state.pass = 1;
@@ -362,19 +378,19 @@ assemble(void)
   /* clean out defines for second pass */
   state.stTopDefines = 
     state.stDefines = push_symbol_table(cmd_defines, state.case_insensitive);
-  if (state.cmd_line.radix != 1)
+  if (!state.cmd_line.radix)
     state.radix = 16;
   state.obj.symbol_num = 0;
   state.obj.section_num = 0;
   state.obj.org_num = 0;
-  state.found_config = 0;
-  state.found_devid = 0;
-  state.found_idlocs = 0;
+  state.found_config = false;
+  state.found_devid = false;
+  state.found_idlocs = false;
   coff_init();
   cod_init();
   lst_init();
   open_src(state.srcfilename, 0);
-  if (gp_debug_disable == 0) {
+  if (!gp_debug_disable) {
     yydebug = 1;
   } else {
     yydebug = 0;
