@@ -1,5 +1,5 @@
 /* Common definitions for gpasm
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman, Craig Franklin
 
 This file is part of gputils.
@@ -43,7 +43,9 @@ enum gpasmValTypes {
   gvt_address,
   gvt_extern,
   gvt_global,
-  gvt_static
+  gvt_static,
+  gvt_debug,
+  gvt_absolute
 };
 
 enum state_types { 
@@ -64,9 +66,6 @@ enum outfile {
 
 enum file_types {
   ft_src,
-  ft_hex,
-  ft_lst,
-  ft_cod,
   ft_other
 };
 
@@ -77,29 +76,31 @@ enum gpasm_modes {
 
 extern struct gpasm_state {
   enum gpasm_modes mode;
+  gp_boolean extended_pic16e;
   int radix;
   enum formats hex_format;
-  int case_insensitive;
-  int quiet;
+  gp_boolean case_insensitive;
+  gp_boolean quiet;
+  gp_boolean use_absolute_path;
   int error_level;		/* 0, 1, 2 */
-  int debug_info;		/* use debug directives for coff outputs */
+  gp_boolean debug_info;	/* use debug directives for coff outputs */
   int path_num;                 /* number of paths in the list */
   char *paths[MAX_PATHS];       /* the list of include paths */
   struct {			/* Command line override flags */
-    int radix;			/* When 1, the value is specified by the */
-    int hex_format;		/* command line */
-    int error_level;
-    int macro_expand;		
-    int processor;
-    int lst_force;
+    gp_boolean radix;		/* value is specified by the command line */
+    gp_boolean hex_format;		
+    gp_boolean error_level;
+    gp_boolean macro_expand;		
+    gp_boolean processor;
+    gp_boolean lst_force;
   } cmd_line;
   int pass;			/* 1 or 2 */
   unsigned int org;		/* Current code-generation point */
-  int dos_newlines;		/* Use DOS newlines in hex file */
-  int memory_dump;		/* Dump instruction memory to standard output */
-  int found_config;		/* config directive in source code */
-  int found_devid;		/* config directive in source code */
-  int found_idlocs;		/* idlocs directive in source code */
+  gp_boolean dos_newlines;	/* Use DOS newlines in hex file */
+  gp_boolean memory_dump;	/* Dump instruction memory to standard output */
+  gp_boolean found_config;	/* config directive in source code */
+  gp_boolean found_devid;	/* config directive in source code */
+  gp_boolean found_idlocs;	/* idlocs directive in source code */
   gp_boolean register_block;    /* detect header register blocks for gp2pub */
   unsigned int maxram;		/* Highest legal memory location */
   enum outfile
@@ -167,18 +168,17 @@ extern struct gpasm_state {
         linetype;
     } line;
     char startdate[80];		/*   When assembly started */
-    int enabled;		/*   nonzero if listing is enabled */
-    int expand;			/*   1 if the macro listings are expanded */
-    int force;                  /*   1 if ignoring nolist directives */
+    gp_boolean enabled;		/*   listing is enabled */
+    gp_boolean expand;		/*   macro listings are expanded */
+    gp_boolean force;		/*   ignoring nolist directives */
     int config_address;		/*   list config address for 16 bit devices */
     char title_name[80];	/*   given in TITLE directive */
     char subtitle_name[80];	/*   given in SUBTITLE directive */
     int tabstop;		/*   tab-stop distance */
-    struct file_context *fc;    /*   Position in the file context stack */
   } lst;
   struct {			/* Symbol file state: */
     FILE *f;			/*   Symbol file output */
-    int enabled;		/*   nonzero if symbol file is enabled */
+    gp_boolean enabled;		/*   nonzero if symbol file is enabled */
     int emitting;               /*   flag indicating when an opcode is emitted */
   } cod;
   struct {			/* Object file state: */
@@ -186,7 +186,7 @@ extern struct gpasm_state {
     gp_section_type *section;	/*   Current section */
     int section_num;		/*   Current section number */
     int org_num;		/*   Current org number */
-    int enabled;		/*   nonzero if object file is enabled */
+    gp_boolean enabled;		/*   nonzero if object file is enabled */
     char new_sec_name[80];	/*   new section name */
     int new_sec_addr;		/*   new section name */
     int new_sec_flags;		/*   new section name */
@@ -224,9 +224,10 @@ struct variable {
 /* Parse node: created by the parser, interpreted by the 'backend' */
 
 struct pnode {
-  enum pnode_tag { constant, symbol, string, list, binop, unop } tag;
+  enum pnode_tag { constant, offset, symbol, string, list, binop, unop } tag;
   union {
     int constant;
+    struct pnode *offset;
     char *symbol;
     struct {
       struct pnode *head, *tail;
