@@ -1,5 +1,5 @@
 /* Some helpful utility functions for gpasm
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    James Bowman, Craig Franklin
 
 This file is part of gputils.
@@ -146,9 +146,9 @@ convert_escape_chars(char *ps, int *value)
 }
 
 void set_global(char *name,
-		gpasmVal value,
-		enum globalLife lifetime,
-		enum gpasmValTypes type)
+                gpasmVal value,
+                enum globalLife lifetime,
+                enum gpasmValTypes type)
 {
   struct symbol *sym;
   struct variable *var;
@@ -209,6 +209,8 @@ void set_global(char *name,
   case gvt_global:
   case gvt_static:
   case gvt_address:
+  case gvt_debug:
+  case gvt_absolute:
     state.obj.symbol_num++;
     break;
   default:
@@ -219,7 +221,7 @@ void set_global(char *name,
 
 void select_errorlevel(int level)
 {
-  if (state.cmd_line.error_level == 1) {
+  if (state.cmd_line.error_level) {
     gpmessage(GPM_SUPVAL, NULL);
   } else {
     if (level == 0) {
@@ -242,15 +244,15 @@ void select_errorlevel(int level)
 
 void select_expand(char *expand)
 {
-  if (state.cmd_line.macro_expand == 1) {
+  if (state.cmd_line.macro_expand) {
     gpmessage(GPM_SUPLIN, NULL);
   } else {
     if (strcasecmp(expand, "ON") == 0) {
-      state.lst.expand = 1;
+      state.lst.expand = true;
     } else if (strcasecmp(expand, "OFF") == 0) {
-      state.lst.expand = 0;
+      state.lst.expand = false;
     } else {
-      state.lst.expand = 1;
+      state.lst.expand = true;
       if (state.pass == 0) {
 	fprintf(stderr,
 		"Error: invalid option \"%s\"\n",
@@ -264,7 +266,7 @@ void select_expand(char *expand)
 
 void select_hexformat(char *format_name)
 {
-  if (state.cmd_line.hex_format == 1) {
+  if (state.cmd_line.hex_format) {
     gpwarning(GPW_CMDLINE_HEXFMT, NULL);
   } else {
     if (strcasecmp(format_name, "inhx8m") == 0) {
@@ -290,7 +292,7 @@ void select_hexformat(char *format_name)
 
 void select_radix(char *radix_name)
 {
-  if (state.cmd_line.radix == 1) {
+  if (state.cmd_line.radix) {
     gpwarning(GPW_CMDLINE_RADIX, NULL);
   } else {
     if (strcasecmp(radix_name, "hex") == 0) {
