@@ -63,6 +63,10 @@ void object_append(gp_object_type *file, char *name)
 
   }
 
+  if (state.optimize.weak_symbols) {
+    gp_cofflink_remove_weak(file);
+  }
+
   return;
 }
 
@@ -97,7 +101,8 @@ void archive_append(gp_archive_type *file, char *name)
    Stop whenever a complete pass through the archive happens and no
    objects are added. */
 
-int scan_index(struct symbol_table *table, gp_archive_type *archive)
+gp_boolean
+scan_index(struct symbol_table *table, gp_archive_type *archive)
 {
   struct symbol *s;
   struct symbol *m;
@@ -107,6 +112,7 @@ int scan_index(struct symbol_table *table, gp_archive_type *archive)
   int num_added = 1; /* initalize to 1 so while loop can be entered */
   char *name;
   char *object_name;
+  gp_boolean modified = false;
 
   while (num_added != 0) {
     num_added = 0;
@@ -129,6 +135,7 @@ int scan_index(struct symbol_table *table, gp_archive_type *archive)
           /* The symbol tables have been modified.  Need to take another
 	     pass to make sure we get everything. */
           num_added++;
+          modified = true;
           free(object_name);
           /* This branch of the table has been modified. Go to the next one */
 	  break;
@@ -137,11 +144,14 @@ int scan_index(struct symbol_table *table, gp_archive_type *archive)
     }
   }
 
-  return 0;
+  return modified;
 }
     
-int scan_archive(gp_archive_type *archive, char *name)
+gp_boolean
+scan_archive(gp_archive_type *archive, char *name)
 {
+  gp_boolean modified;
+
   state.symbol.archive = push_symbol_table(NULL, false);
 
   /* If necessary, build a symbol index for the archive. */
@@ -160,11 +170,11 @@ int scan_archive(gp_archive_type *archive, char *name)
 
   /* Scan the symbol index for symbols in the missing symbol table.  If
      found, add the object to state.objects. */
-  scan_index(state.symbol.archive, archive);
+  modified = scan_index(state.symbol.archive, archive);
 
   state.symbol.archive = pop_symbol_table(state.symbol.archive);
 
-  return 0;
+  return modified;
 }
 
 /* Remove a symbol the linker created from the missing table */
@@ -235,10 +245,12 @@ search_idata(void)
 void build_tables(void)
 {
   gp_object_type *list = state.object;
-  struct archivelist *arlist = state.archives;
+  struct archivelist *arlist;
+  gp_boolean modified;
   int i;
   struct symbol *s;
   char *name;
+  gp_coffsymbol_type *var;
 
   /* Create the object file symbol tables */  
   while (list != NULL) {
@@ -250,14 +262,29 @@ void build_tables(void)
 
   /* All of the objects have been scanned.  If there are remaining references
      to symbols, then the archives must contain the missing references. */
-  if (count_missing()) {
-    while (arlist != NULL) {
-      scan_archive(arlist->archive, arlist->name);
+  if (count_missing() && (state.archives != NULL)) {
+    modified = false;
+    arlist = state.archives;
+    while (1) {
+      if (scan_archive(arlist->archive, arlist->name)) {
+        modified = true;
+      }
       if (count_missing() == 0) {
         /* No more missing references, no need to continue. */
         break;
+      } else if (arlist->next == NULL) {
+        if (modified) {
+          /* At least one object was loaded from an archive and there are
+             still missing symbols.  Scan all the archives again. */
+          modified = false;
+          arlist = state.archives;
+        } else {
+          /* Quit */
+          break;
+        }
+      } else {
+        arlist = arlist->next;
       }
-      arlist = arlist->next;
     }
   }
 
@@ -279,8 +306,12 @@ void build_tables(void)
       for (s = state.symbol.missing->hash_table[i]; s; s = s->next) {
         name = get_symbol_name(s);
         assert(name != NULL);
-        gp_error("missing definition for symbol \"%s\"", name);
-      }      
+        var = get_symbol_annotation(s);
+        assert(var != NULL);
+        gp_error("missing definition for symbol \"%s\", required by \"%s\"",
+                 name,
+                 var->file->filename);
+      }
     }
     exit(1);
   }
@@ -347,7 +378,31 @@ void gplink_open_coff(char *name)
 
 }   
 
-#define GET_OPTIONS "?a:cdf:hI:lmo:qrs:t:v"
+static void
+set_optimize_level(void)
+{
+
+  /* default */
+  state.optimize.weak_symbols = false;
+
+  switch(state.optimize.level) {
+  case 3:
+    /* fall through */
+  case 2:
+    /* fall through */
+  case 1:
+    state.optimize.weak_symbols = true;
+    /* fall through */
+  case 0:
+    break;
+  default:
+    gp_error("invalid optimization level");
+  }
+
+  return;
+}
+
+#define GET_OPTIONS "?a:cdf:hI:lmo:O:qrs:t:v"
 
   static struct option longopts[] =
   {
@@ -360,6 +415,7 @@ void gplink_open_coff(char *name)
     { "no-list",     0, 0, 'l' },
     { "map",         0, 0, 'm' },
     { "output",      1, 0, 'o' },
+    { "optimize",    1, 0, 'O' },
     { "quiet",       0, 0, 'q' },
     { "use-shared",  0, 0, 'r' },
     { "script",      1, 0, 's' },
@@ -380,6 +436,7 @@ init(void)
   state.numpaths = 0;
   state.byte_addr = 0;
   state.processor = no_processor;
+  state.optimize.level = 1;
   state.codfile = normal;
   state.hexfile = normal;
   state.lstfile = normal;
@@ -428,6 +485,7 @@ void show_usage(void)
   printf("  -l, --no-list                  Disable list file output.\n");
   printf("  -m, --map                      Output a map file.\n");
   printf("  -o FILE, --output FILE         Alternate name of output file.\n");
+  printf("  -O OPT, --optimize OPT         Optimization level [1].\n");
   printf("  -q, --quiet                    Quiet.\n");
   printf("  -r, --use-shared               Use shared memory if necessary.\n");
   printf("  -s FILE, --script FILE         Linker script.\n");
@@ -461,6 +519,21 @@ process_args( int argc, char *argv[])
   gp_boolean usage = false;
   char *pc;
 
+  /* first pass through options */
+  while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, 0)) != EOF) {
+    switch (c) {
+    case 'O':
+      state.optimize.level = atoi(optarg);
+      break;
+    }
+  }
+
+  /* reset the getopt_long index for the next call */
+  optind = 1;
+
+  set_optimize_level();
+
+  /* second pass through options */
   while ((c = getopt_long(argc, argv, GET_OPTIONS, longopts, 0)) != EOF) {
     switch (c) {
     case 'a':
@@ -511,6 +584,9 @@ process_args( int argc, char *argv[])
       if (pc)
         *pc = 0;
       break;
+    case 'O':
+      /* do nothing */
+      break;
     case 'q':
       gp_quiet = true;
       break;
@@ -629,6 +705,10 @@ linker(void)
     state.byte_addr = 0;
   }
 
+  /* An error occured while reading the input files, no need to continue */
+  if (gp_num_errors)
+    return EXIT_FAILURE;
+
   /* Construct the symbol tables. Determine which archive members are 
      required to resolve external references. */
   build_tables();  
