@@ -126,6 +126,63 @@ gp_link_add_symbols(struct symbol_table *definition,
   return 0;
 }
 
+/* Determine if any relocation uses the symbol. */
+
+static gp_boolean
+_has_relocation(gp_object_type *object, gp_symbol_type *symbol)
+{
+  gp_section_type *section;
+  gp_reloc_type *relocation;
+
+  section = object->sections;
+  while (section != NULL) {
+    relocation = section->relocations;
+    while (relocation != NULL) {
+      if (relocation->symbol == symbol) {
+        return true;
+      }
+      relocation = relocation->next;
+    }
+    section = section->next;
+  }
+
+  return false;
+}
+
+/* Remove any weak symbols in the object. */
+
+void
+gp_cofflink_remove_weak(gp_object_type *object)
+{
+  gp_symbol_type *symbol;
+  gp_symbol_type *previous = NULL;
+
+  gp_debug("removing weak symbols from %s", object->filename);
+
+  /* Search the symbol table for extern symbols. */
+  symbol = object->symbols;
+  while (symbol != NULL) {
+    if ((symbol->class == C_EXT) && 
+        (symbol->section_number == N_UNDEF) &&
+        (!_has_relocation(object, symbol))) {
+      gp_debug("  removed weak symbol \"%s\"", symbol->name);
+      if (previous == NULL) {
+        /* removing first symbol in the list */
+        object->symbols = object->symbols->next;
+      } else {
+        previous->next = symbol->next;
+      }
+      object->num_symbols--;    
+    } else {
+      previous = symbol;
+    }
+
+    symbol = symbol->next;
+  }
+
+  return;
+}
+
 /* Combine all sections and symbols from all objects into one object file. */
 
 void
@@ -437,6 +494,11 @@ gp_cofflink_merge_sections(gp_object_type *object, int byte_addr)
       while(list != NULL) {
         if (list->next == second) {
           list->next = second->next;
+          if (list->next == NULL) {
+            /* The last section in the list is being removed, so update
+               the tail. */ 
+            object->sections_tail = list;
+          }
           break;
         }
         list = list->next;
@@ -1360,6 +1422,9 @@ gp_cofflink_patch_addr(enum proc_class class,
       write_data = 0;
     }
     break;
+  case RELOCT_ALL:
+    data = value & 0xffff;   
+    break; 
   case RELOCT_IBANKSEL:
     switch(class) {
     case PROC_CLASS_PIC14:
@@ -1435,12 +1500,12 @@ gp_cofflink_patch_addr(enum proc_class class,
     break;
   case RELOCT_BRA:
     offset = (value - ((org + 1) << 1)) >> 1;
-    check_relative(section, org, offset, 0x7ff);
+    check_relative(section, org, offset, 0x3ff);
     data = offset & 0x7ff;
     break;
   case RELOCT_CONDBRA:
     offset = (value - ((org + 1) << 1)) >> 1;
-    check_relative(section, org, offset, 127);
+    check_relative(section, org, offset, 0x7f);
     data = offset & 0xff;
     break;
   case RELOCT_UPPER:
@@ -1470,7 +1535,6 @@ gp_cofflink_patch_addr(enum proc_class class,
     break;
   /* unimplemented relocations */
   case RELOCT_PAGESEL:
-  case RELOCT_ALL:
   case RELOCT_SCNSZ_LOW:
   case RELOCT_SCNSZ_HIGH:
   case RELOCT_SCNSZ_UPPER:
@@ -1579,7 +1643,7 @@ gp_cofflink_clean_table(gp_object_type *object)
 
   while (symbol != NULL) {
     if ((symbol->class == C_EXT) && 
-        (symbol->section_number == 0)) {
+        (symbol->section_number == N_UNDEF)) {
       /* This is an external symbol defined elsewhere */
       if (previous == NULL) {
         /* removing first symbol in the list */
